/* nmalloc.c
   Module for allocation wrapping routines. */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_STRING_H
#  include <string.h>
#else
#  include <strings.h>
#endif /* HAVE_STRING_H */

#include "nmalloc.h"

extern FILE *lfile;
extern int quiet;

/* nmalloc, nrealloc and nstrdup exit the program if there is not
   enough memory. nstrdup also implements strdup on systems that
   do not have it. */
void *
nmalloc(size_t size)
{
   void *res;

   res = malloc(size);
   if (!res)
      memfatal("malloc");
   return res;
}

void *
nrealloc(void *obj, size_t size)
{
   void *res;

   /* Not all Unixes have the feature of realloc() that calling it
      with a NULL-pointer is the same as malloc(), but it is easy to
      simulate. */
   if (obj)
      res = realloc(obj, size);
   else
      res = malloc(size);
   if (!res)
      memfatal("realloc");
   return res;
}

char *
nstrdup(char *s)
{
   char *s1;

#ifndef HAVE_STRDUP
   int l;
   
   l = strlen(s);
   s1 = malloc(l + 1);
   if (!s1)
      memfatal("strdup");
   memcpy(s1, s, l + 1);
   return s1;
#else
   s1 = strdup(s);
   if (!s1)
      memfatal("strdup");
   return s1;
#endif
}

void
memfatal(char *s)
{
   if (!quiet)
      fprintf(lfile, "%s: Not enough memory.\n", s);
   exit(1);
}

