/* url.h
   Header file for URL handling */

#ifndef URL_H
#define URL_H

/* URL separator (for findurl) */
#ifndef URL_SEPARATOR
#  define URL_SEPARATOR "!\"#'()*+,>`{}|<>"
#endif /* URL_SEPARATOR */

/* Default port definitions */
#define DEFAULT_HTTP_PORT 80
#define DEFAULT_FTP_PORT 21

/* Buffer for file-loading */
#ifndef FILE_BUFFER_SIZE
#  define FILE_BUFFER_SIZE 1024
#endif /* FILE_BUFFER_SIZE */

/* ASCII char -> HEX digit */
#define ASC2HEXD(x) (((x) >= '0' && (x) <= '9') ? \
		     ((x) - '0') : (((x) & 223) - 'A' + 10))

/* HEX digit -> ASCII char */
#define HEXD2ASC(x) (((x) >= 0 && (x) <= 9) ? \
		     ((x) + '0') : ((x) - 10 + 'A'))

/* A list of unsafe characters for encoding, as per RFC1738.  '@' and
   ':' (not listed in RFC) were added because of user/password
   encoding, and \033 because of safe printing. */
#define URL_UNSAFE " <>\"#%{}|\\^~[]`@:\033"

/* Just another ugly macro (defines a static local char *). */
#define URL_CLEANSE(s) if (contains_unsafe(s)) \
{ static char *tmp; tmp = encode_string(s); free(s); s = tmp; }

typedef struct proto {
   char *name;
   uerr_t ind;
   int port;
} proto_t;

/* Function declarations */
int skip_url(char *);
int contains_unsafe(char *);
void decode_string(char *);
char *encode_string(char *);
uerr_t urlproto(char *);
uerr_t parseurl(char *, char *, int *, char *);
uerr_t parse_uname(char *, char **, char **);
char *hide_url(char *);
char *findurl(char *, int, int *);
int mymkdir(char *);
char *mkstruct(char *);
char *url_filename(char *, char *);
char *unique_name(char *, int);
int exists(char *);
char *construct(char *, char *, int, int);
void path_simplify(char *);
char *opt_url(char *, int);
int numdigit(int);
int acceptable(char *);
int in_acclist(char **, char *);
char *suffix(char *s);
int matches(char **, char *);
char **sepstring(char *);
void load_file(FILE *, char **, long *);
void create_hash(char ***, char **);
int in_hash(char **, char *);
void add_hash(char ***, char *);
void freelists(void);
char *getproxy(uerr_t);
int no_proxy_match(char *, char *);
void parse_dir(char *, char **, char **);

#endif /* URL_H */
